#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#define INPUT "policie.in"
#define OUTPUT "policie.out"

#define MAXN 100000

FILE *in, *out;		/* Vstupni a vystupni soubor */
int n, p;		/* Pocet vetveni a pocet mafianu */
int deg[MAXN];		/* Pocet stok vedoucich z vetveni */
int e[MAXN][2];		/* Seznam stok */
int edge_ptr[MAXN];	/* Cislo v neib[], kde zacinaji sousede kazdeho vetveni */
int neib[2*MAXN];	/* Seznamy sousedu vetveni */
char podezrely[MAXN];	/* Je do daneho vetveni pripojen dum mafiana? */
int hlidek;		/* Potrebny pocet hlidek */

void nacti_vstup(void)
{
  int i, a, b;
  int tmp_ptr[MAXN];

  /* Nacteme stoky */
  fscanf(in, "%d %d", &n, &p);
  if (n<3 || n>MAXN || p<2 || p>= n) {
    puts("Vstup nesouhlasi se zadanim.");
    exit(1);
  }
  memset(deg, 0, n*sizeof(int));
  for (i = 0; i < n-1; i++) {
    fscanf(in, "%d %d", &a, &b);
    if (a<1 || a>n || b < 1 || b>n || a==b) {
      puts("Popis grafu nesouhlasi se zadanim.");
      exit(1);
    }
    a--; b--;
    e[i][0] = a;
    e[i][1] = b;
    deg[a]++;
    deg[b]++;
  }
  /* Vytvorime seznamy sousedu a nastavime spravne jejich pocatky */
  edge_ptr[0] = 0;
  for (i = 1; i < n; i++)
    edge_ptr[i] = edge_ptr[i-1]+deg[i-1];
  memset(tmp_ptr, 0, n*sizeof(int));
  for (i = 0; i < n-1; i++) {
    neib[edge_ptr[e[i][0]]+tmp_ptr[e[i][0]]++] = e[i][1];
    neib[edge_ptr[e[i][1]]+tmp_ptr[e[i][1]]++] = e[i][0];
  }

  /* Nacteme seznam mafianu */
  for (i = 0; i < p; i++) {
    fscanf(in, "%d", &a);
    if (a < 1 || a > n) {
      puts("Spatne cislo podezreleho!");
      exit(1);
    }
    a--;
    if (podezrely[a]) {
      puts("Podezrely vicekrat na vstupu!");
      exit(1);
    }
    podezrely[a] = 1;
  }
}

/* Spocte pocet hlidek pro T_v, vrati 1 pokud T_v nema reseni strazici 'v'. */
int hledej(int v, int rodic)
{
  int i, nehlidanych = 0;

  /* Projdeme vsechny sousedy, ze kterych k nam vede stoka, a zjistime situaci */
  for (i = 0; i < deg[v]; i++)
    if (neib[edge_ptr[v]+i] != rodic)
      nehlidanych += hledej(neib[edge_ptr[v]+i], v);
  /* Musime vetveni $v$ hlidat? */
  /* To je treba bud pokud mame alespon 2 sousedy nemajici opt. reseni strazici 'v'
   * (pak je treba izolovat mafiany z nich), nebo pokud ve 'v' sidli mafian
   * a mame alespon jednoho souseda nemajiciho opt. reseni strazici 'v'. */
  if (nehlidanych > 1 || (podezrely[v] && nehlidanych > 0)) {
      hlidek++;
      return 0;
  }
  /* Jinak nemusime hlidat, jen vratime, jestli je T_v nema reseni strazici 'v' */
  return nehlidanych || podezrely[v];
}

int main(void)
{
  if (!(in = fopen(INPUT, "r"))) {
    puts("Nemuzu otevrit vstupni soubor.");
    return 1;
  }
  if (!(out = fopen(OUTPUT, "w"))) {
    puts("Nemuzu otevrit vystupni soubor.");
    return 1;
  }
  nacti_vstup();
  hledej(0,0);
  fprintf(out, "%d\n", hlidek);
  return 0;
}
